@@ -8,6 +8,11 @@
 //! [`ChdHeader`](crate::header::ChdHeader) makes no ABI guarantees and is not ABI-compatible
 //! with [`libchdr::chd_header`](https://github.com/rtissera/libchdr/blob/6eeb6abc4adc094d489c8ba8cafdcff9ff61251b/include/libchdr/chd.h#L302).
 
+#[cfg(feature = "avhuff")]
+use crate::compression::codecs::AVHuffCodec;
+use crate::compression::codecs::{
+    CdFlCodec, CdLzCodec, CdZlCodec, HuffmanCodec, LzmaCodec, NoneCodec, RawFlacCodec, ZlibCodec,
+};
 use crate::compression::{CompressionCodec, InternalCodec};
 use crate::error::{ChdError, Result};
 use crate::make_tag;
@@ -19,7 +24,6 @@ use num_traits::FromPrimitive;
 use regex::bytes::Regex;
 use std::ffi::CStr;
 use std::io::{Cursor, Read, Seek, SeekFrom};
-use crate::compression::codecs::{CdFlCodec, CdLzCodec, CdZlCodec, NoneCodec, ZlibCodec};
 
 /// The types of compression codecs supported in a CHD file.
 #[repr(u32)]
@@ -41,6 +45,14 @@ pub enum CodecType {
     LzmaCdV5 = make_tag(b"cdlz"),
     /// V5 CD FLAC compression (cdfl)
     FlacCdV5 = make_tag(b"cdfl"),
+    /// V5 FLAC compression (flac)
+    FlacV5 = make_tag(b"flac"),
+    /// V5 LZMA compression (lzma)
+    LzmaV5 = make_tag(b"lzma"),
+    /// V5 AV/Huffman compression (avhu)
+    AVHuffV5 = make_tag(b"avhu"),
+    /// V5 Huffman compression
+    HuffV5 = make_tag(b"huff"),
 }
 
 impl CodecType {
@@ -60,10 +72,12 @@ impl CodecType {
     /// * Zlib/Zlib+/Zlib V5
     /// * CDZL (CD Zlib)
     /// * CDLZ (CD LZMA)
-    /// * CDFL (CD Flac)
+    /// * CDFL (CD FLAC)
+    /// * FLAC (Raw FLAC)
+    /// * LZMA (Raw LZMA)
     ///
-    /// AVHuff decompression is not supported.
-    pub fn init(&self, hunk_size: u32) -> Result<Box<dyn CompressionCodec>> {
+    /// AVHuff decompression is experimental and can be enabled with the `avhuff` feature.
+    pub(crate) fn init(&self, hunk_size: u32) -> Result<Box<dyn CompressionCodec>> {
         match self {
             CodecType::None => {
                 NoneCodec::new(hunk_size).map(|x| Box::new(x) as Box<dyn CompressionCodec>)
@@ -80,7 +94,21 @@ impl CodecType {
             CodecType::FlacCdV5 => {
                 CdFlCodec::new(hunk_size).map(|x| Box::new(x) as Box<dyn CompressionCodec>)
             }
-            CodecType::AV => return Err(ChdError::UnsupportedFormat),
+            CodecType::LzmaV5 => {
+                LzmaCodec::new(hunk_size).map(|x| Box::new(x) as Box<dyn CompressionCodec>)
+            }
+            CodecType::FlacV5 => {
+                RawFlacCodec::new(hunk_size).map(|x| Box::new(x) as Box<dyn CompressionCodec>)
+            }
+            CodecType::HuffV5 => {
+                HuffmanCodec::new(hunk_size).map(|x| Box::new(x) as Box<dyn CompressionCodec>)
+            }
+            #[cfg(feature = "avhuff")]
+            CodecType::AV | CodecType::AVHuffV5 => {
+                AVHuffCodec::new(hunk_size).map(|x| Box::new(x) as Box<dyn CompressionCodec>)
+            }
+            #[allow(unreachable_patterns)]
+            _ => Err(ChdError::UnsupportedFormat),
         }
     }
 }
@@ -343,7 +371,7 @@ impl ChdHeader {
     }
 
     /// Returns the size of each hunk in the CHD file in bytes.
-    pub const fn hunk_bytes(&self) -> u32 {
+    pub const fn hunk_size(&self) -> u32 {
         match self {
             ChdHeader::V1Header(c) => c.hunk_bytes,
             ChdHeader::V2Header(c) => c.hunk_bytes,
@@ -411,26 +439,26 @@ impl ChdHeader {
     pub(crate) fn create_compression_codecs(&self) -> Result<Vec<Box<dyn CompressionCodec>>> {
         match self {
             ChdHeader::V1Header(c) => CodecType::from_u32(c.compression)
-                .map(|e| e.init(self.hunk_bytes()))
+                .map(|e| e.init(self.hunk_size()))
                 .ok_or(ChdError::UnsupportedFormat)?
                 .map(|e| vec![e]),
             ChdHeader::V2Header(c) => CodecType::from_u32(c.compression)
-                .map(|e| e.init(self.hunk_bytes()))
+                .map(|e| e.init(self.hunk_size()))
                 .ok_or(ChdError::UnsupportedFormat)?
                 .map(|e| vec![e]),
             ChdHeader::V3Header(c) => CodecType::from_u32(c.compression)
-                .map(|e| e.init(self.hunk_bytes()))
+                .map(|e| e.init(self.hunk_size()))
                 .ok_or(ChdError::UnsupportedFormat)?
                 .map(|e| vec![e]),
             ChdHeader::V4Header(c) => CodecType::from_u32(c.compression)
-                .map(|e| e.init(self.hunk_bytes()))
+                .map(|e| e.init(self.hunk_size()))
                 .ok_or(ChdError::UnsupportedFormat)?
                 .map(|e| vec![e]),
             ChdHeader::V5Header(c) => c
                 .compression
                 .map(CodecType::from_u32)
                 .map(|f| f.ok_or(ChdError::UnsupportedFormat))
-                .map(|f| f.and_then(|f| f.init(self.hunk_bytes())))
+                .map(|f| f.and_then(|f| f.init(self.hunk_size())))
                 .into_iter()
                 .collect(),
         }
@@ -463,7 +491,7 @@ impl ChdHeader {
         }
 
         // require valid hunk size
-        if self.hunk_bytes() == 0 || self.hunk_bytes() >= 65536 * 256 {
+        if self.hunk_size() == 0 || self.hunk_size() >= 65536 * 256 {
             return false;
         }
 
